import speech_recognition as sr
import pyttsx3
import datetime
import wikipedia
import webbrowser
import os
import requests # For checking website accessibility, if you want to get advanced

# Initialize speech recognizer and text-to-speech engine
r = sr.Recognizer()
engine = pyttsx3.init()

def speak(text):
    engine.say(text)
    engine.runAndWait()

def listen():
    with sr.Microphone() as source:
        print("Listening...")
        # Adjusting the recognizer's energy threshold can help with background noise
        r.adjust_for_ambient_noise(source, duration=0.5) 
        r.pause_threshold = 1 # seconds of non-speaking audio before a phrase is considered complete
        audio = r.listen(source)
    try:
        print("Recognizing...")
        # Using Google's online recognition, which is generally quite good
        query = r.recognize_google(audio, language='en-in') 
        print(f"User said: {query}\n")
    except sr.UnknownValueError:
        print("Sorry, I could not understand what you said.")
        return "None" # Return "None" for unrecognized speech
    except sr.RequestError as e:
        print(f"Could not request results from Google Speech Recognition service; {e}")
        speak("I'm having trouble connecting to the speech recognition service. Please check your internet connection.")
        return "None"
    except Exception as e:
        # Catch any other unexpected errors during speech recognition
        print(f"An unexpected error occurred during listening: {e}")
        speak("An error occurred while trying to listen. Please try again.")
        return "None"
    return query

def wish_me():
    hour = int(datetime.datetime.now().hour)
    if 0 <= hour < 12:
        speak("Good Morning!")
    elif 12 <= hour < 18:
        speak("Good Afternoon!")
    else:
        speak("Good Evening!")
    
    # Using the current location for a more personalized greeting
    speak("I am Torus from Deoria. How can I help you?")

# --- Main execution loop ---
if __name__ == "__main__":
    wish_me()
    while True:
        query = listen().lower()

        # If listen() returns "None", it means no understandable speech was detected.
        # We should skip processing the query in this case.
        if query == "none":
            continue 

        command_recognized = False # Flag to track if any command was handled

        # Logic for executing tasks based on query
        if 'wikipedia' in query:
            command_recognized = True
            speak('Searching Wikipedia...')
            try:
                query = query.replace("wikipedia", "").strip()
                if not query: # Handle cases like "Wikipedia" with no search term
                    speak("Please tell me what you want to search on Wikipedia.")
                    continue
                results = wikipedia.summary(query, sentences=2)
                speak(f"According to Wikipedia, {results}")
                print(results)
            except wikipedia.exceptions.PageError:
                speak(f"Sorry, I couldn't find anything on Wikipedia for {query}.")
            except wikipedia.exceptions.DisambiguationError as e:
                speak(f"There are multiple results for {query}. Please be more specific.")
                print(f"Disambiguation options: {e.options[:5]}...") # Show a few options
            except Exception as e:
                speak(f"An error occurred while searching Wikipedia: {e}")

        elif 'open youtube' in query:
            command_recognized = True
            try:
                webbrowser.open("https://www.youtube.com") # Use https for security
                speak("Opening YouTube.")
            except Exception as e:
                speak(f"Sorry, I couldn't open YouTube. Error: {e}")

        elif 'open google' in query:
            command_recognized = True
            try:
                webbrowser.open("https://www.google.com") # Use https
                speak("Opening Google.")
            except Exception as e:
                speak(f"Sorry, I couldn't open Google. Error: {e}")

        elif 'the time' in query:
            command_recognized = True
            str_time = datetime.datetime.now().strftime("%H:%M:%S")
            speak(f"Sir, the time is {str_time}")
        
        elif 'the date' in query:
            command_recognized = True
            str_date = datetime.datetime.now().strftime("%A, %B %d, %Y")
            speak(f"Today is {str_date}")

        # Adding a specific query for weather in Deoria
        elif 'weather in deoria' in query:
            command_recognized = True
            speak("I can tell you the current weather in Deoria. Please allow me a moment to fetch the information.")
            # You would integrate an API call here (e.g., OpenWeatherMap)
            # For now, let's give a placeholder response
            speak("I'm sorry, I don't have direct access to real-time weather data yet. But I can tell you that today is Thursday, June 5, 2025.")

        elif 'open gemini' in query:
            command_recognized = True
            try:
                webbrowser.open("https://gemini.google.com") # Corrected URL for Gemini
                speak("Opening Gemini.")
            except Exception as e:
                speak(f"Sorry, I couldn't open Gemini. Error: {e}")

        elif 'open chatgpt' in query or 'open chat' in query:
            command_recognized = True
            try:
                webbrowser.open("https://chat.openai.com") # Corrected URL for ChatGPT
                speak("Opening ChatGPT.")
            except Exception as e:
                speak(f"Sorry, I couldn't open ChatGPT. Error: {e}")
        
        elif 'gaming by anagh' in query or 'gaming by alag' in query:
            command_recognized = True
            try:
                webbrowser.open("https://gaming-by-anagh.neocities.org")
                speak("Opening Gaming by Anagh.")
            except Exception as e:
                speak(f"Sorry, I couldn't open Gaming by Anagh. Error: {e}")
        
        elif 'coding by anagh' in query or 'coding by alag' in query:
            command_recognized = True
            try:
                webbrowser.open("https://coding-by-anagh.neocities.org")
                speak("Opening Coding by Anagh.")
            except Exception as e:
                speak(f"Sorry, I couldn't open Coding by Anagh. Error: {e}")

        elif 'open code' in query:
            command_recognized = True
            # This path is highly specific to your machine. 
            # Make sure it's correct or use an environment variable.
            code_path = "C:\\Users\\YourUsername\\AppData\\Local\\Programs\\Microsoft VS Code\\Code.exe" 
            try:
                if os.path.exists(code_path):
                    os.startfile(code_path)
                    speak("Opening VS Code.")
                else:
                    speak("I can't find VS Code at the specified path. Please check the path in my code.")
            except Exception as e:
                speak(f"Sorry, I couldn't open VS Code. Error: {e}")

        elif 'exit' in query or 'stop' in query or 'quit' in query:
            command_recognized = True
            speak("Goodbye!")
            break
        
        # --- Catch-all for unrecognized commands ---
        if not command_recognized:
            speak("The thing you are asking is not in my capacity. Thank you.")